package com.ningdatech.kqapi.admin.manage;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ningdatech.basic.model.IdVo;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.kqapi.admin.model.req.MatterListReq;
import com.ningdatech.kqapi.admin.model.vo.MatterDetailDTO;
import com.ningdatech.kqapi.admin.model.vo.MatterListVO;
import com.ningdatech.kqapi.zzsfw.model.entity.KqZzsfwMenu;
import com.ningdatech.kqapi.zzsfw.service.IKqZzsfwMenuService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * <p>
 * MatterManage
 * </p>
 *
 * @author WendyYang
 * @since 15:41 2024/4/20
 */
@Component
@RequiredArgsConstructor
public class AdminMatterManage {

    private final IKqZzsfwMenuService zzsfwMenuService;

    public PageVo<MatterListVO> pageMatter(MatterListReq req) {
        Wrapper<KqZzsfwMenu> query = Wrappers.lambdaQuery(KqZzsfwMenu.class)
                .like(StrUtil.isNotBlank(req.getMatterName()), KqZzsfwMenu::getItemName, req.getMatterName())
                .eq(req.getZoneId() != null, KqZzsfwMenu::getZoneId, req.getZoneId())
                .eq(req.getWindowId() != null, KqZzsfwMenu::getWindowId, req.getWindowId())
                .eq(req.getMatterType() != null, KqZzsfwMenu::getType, req.getMatterType())
                .eq(req.getDepartment() != null, KqZzsfwMenu::getDepartment, req.getDepartment())
                .orderByDesc(KqZzsfwMenu::getCreateOn);
        Page<KqZzsfwMenu> page = zzsfwMenuService.page(req.page(), query);
        if (page.getTotal() == 0) {
            return PageVo.empty();
        }
        List<MatterListVO> data = CollUtils.convert(page.getRecords(),
                w -> MatterListVO.builder()
                        .matterName(w.getItemName())
                        .matterType(w.getType())
                        .department(w.getDepartment())
                        .zoneName(w.getZoneName())
                        .windowName(w.getWindow())
                        .build());
        return PageVo.of(data, page.getTotal());
    }

    public void del(IdVo<Long> id) {
        zzsfwMenuService.removeById(id.getId());
    }

    public MatterDetailDTO detail(Long id) {
        KqZzsfwMenu matter = zzsfwMenuService.getById(id);
        if (matter == null) {
            return null;
        }
        return MatterDetailDTO.builder()
                .id(matter.getId())
                .free(matter.getFree())
                .onlinePay(matter.getOnlinePay())
                .hasUrl(matter.getHasUrl())
                .matterType(matter.getType())
                .matterName(matter.getItemName())
                .department(matter.getDepartment())
                .handleAddress(matter.getHandleAddress())
                .handleChannel(matter.getHandleChannel())
                .handleTime(matter.getHandleTime())
                .implSubjectNature(matter.getImplSubjectNature())
                .publicityMaterials(matter.getPublicityMaterials())
                .sceneDesc(matter.getSceneDesc())
                .serviceContent(matter.getServiceContent())
                .serviceObject(matter.getServiceObject())
                .serviceProcess(matter.getServiceProcess())
                .sort(matter.getSort())
                .telephone(matter.getTelephone())
                .window(matter.getWindow())
                .windowId(matter.getWindowId())
                .zoneId(matter.getZoneId())
                .zoneName(matter.getZoneName())
                .webApplyUrl(matter.getWebapplyurl())
                .build();
    }

    public void saveOrUpdate(MatterDetailDTO req) {
        KqZzsfwMenu matter = new KqZzsfwMenu();
        matter.setId(req.getId());
        matter.setFree(req.getFree());
        matter.setOnlinePay(req.getOnlinePay());
        matter.setHasUrl(req.getHasUrl());
        matter.setType(req.getMatterType());
        matter.setItemName(req.getMatterName());
        matter.setDepartment(req.getDepartment());
        matter.setHandleAddress(req.getHandleAddress());
        matter.setHandleChannel(req.getHandleChannel());
        matter.setHandleTime(req.getHandleTime());
        matter.setImplSubjectNature(req.getImplSubjectNature());
        matter.setPublicityMaterials(req.getPublicityMaterials());
        matter.setSceneDesc(req.getSceneDesc());
        matter.setServiceContent(req.getServiceContent());
        matter.setServiceObject(req.getServiceObject());
        matter.setServiceProcess(req.getServiceProcess());
        matter.setSort(req.getSort());
        matter.setTelephone(req.getTelephone());
        matter.setWindow(req.getWindow());
        matter.setWindowId(req.getWindowId());
        matter.setZoneId(req.getZoneId());
        matter.setZoneName(req.getZoneName());
        matter.setWebapplyurl(req.getWebApplyUrl());
        zzsfwMenuService.saveOrUpdate(matter);
    }

}
