package com.ningdatech.kqapi.zzsfw.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.Assert;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.ningdatech.kqapi.zzsfw.constants.ZzsfwMenuConstant;
import com.ningdatech.kqapi.zzsfw.model.dto.KqZzsfwMattersDeduplicateDTO;
import com.ningdatech.kqapi.zzsfw.model.dto.KqZzsfwMenuDTO;
import com.ningdatech.kqapi.zzsfw.model.entity.KqZzsfwMenu;
import com.ningdatech.kqapi.zzsfw.model.entity.KqZzsfwMattersDeduplicate;
import com.ningdatech.kqapi.zzsfw.model.vo.MatterTopVO;
import com.ningdatech.kqapi.zzsfw.model.vo.MattersVO;
import com.ningdatech.kqapi.zzsfw.model.vo.TreeVO;
import com.ningdatech.kqapi.zzsfw.enumeration.ItemTypeEnum;
import com.ningdatech.kqapi.zzsfw.service.IKqZzsfwMatterDeduplicateService;
import com.ningdatech.kqapi.zzsfw.service.IKqZzsfwMenuService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.Statement;
import java.text.DecimalFormat;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Classname MatterManage
 * @Description
 * @Date 2023/10/25 14:55
 * @Author PoffyZhang
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class MatterManage {

    @Value("${spring.datasource.url}")
    public String url;

    @Value("${spring.datasource.username}")
    public String username;

    @Value("${spring.datasource.password}")
    public String password;

    private final IKqZzsfwMenuService menuService;

    private final IKqZzsfwMatterDeduplicateService matterDeduplicateService;

    public List<TreeVO> getMatters() {
        List<KqZzsfwMenu> matters = menuService.list(Wrappers.lambdaQuery(KqZzsfwMenu.class)
                .orderByAsc(KqZzsfwMenu::getSort));
        if(CollUtil.isEmpty(matters)){
            return Collections.emptyList();
        }

        //如果是政府类型的 就隐藏 没有链接的
        List<KqZzsfwMenu> finalMatters = matters.stream().filter(m -> Objects.nonNull(m.getType()) &&
                (!m.getType().equals(ItemTypeEnum.GOV.getCode()) || Objects.nonNull(m.getWebapplyurl())))
                .collect(Collectors.toList());
        Set<String> zoneSet = new HashSet<>();
        List<TreeVO> res = finalMatters.stream().filter(m -> zoneSet.add(m.getZoneName())).map(m -> {
            TreeVO zone = new TreeVO();
            zone.setName(m.getZoneName());
            zone.setChilren(generateWindows(m, finalMatters));
            zone.setType(ZzsfwMenuConstant.MENU_TYPE_ZONE);
            return zone;
        }).collect(Collectors.toList());
        return res;
    }

    /**
     * 查询社区
     * @return
     */
    public List<TreeVO> getZones() {
        List<KqZzsfwMenu> matters = menuService.list(Wrappers.lambdaQuery(KqZzsfwMenu.class)
                .orderByAsc(KqZzsfwMenu::getSort));
        if(CollUtil.isEmpty(matters)){
            return Collections.emptyList();
        }

        //如果是政府类型的 就隐藏 没有链接的
        List<KqZzsfwMenu> finalMatters = matters.stream().filter(m -> Objects.nonNull(m.getType()) &&
                        (!m.getType().equals(ItemTypeEnum.GOV.getCode()) || Objects.nonNull(m.getWebapplyurl())))
                .collect(Collectors.toList());
        Set<String> zoneSet = new HashSet<>();
        return finalMatters.stream().filter(m -> zoneSet.add(m.getZoneName())).map(m -> {
            TreeVO zone = new TreeVO();
            zone.setName(m.getZoneName());
            zone.setType(ZzsfwMenuConstant.MENU_TYPE_ZONE);
            return zone;
        }).collect(Collectors.toList());
    }

    public List<TreeVO> getWindows(String zoneName) {
        List<KqZzsfwMenu> matters = menuService.list(Wrappers.lambdaQuery(KqZzsfwMenu.class)
                .eq(StringUtils.isNotBlank(zoneName), KqZzsfwMenu::getZoneName,zoneName)
                .orderByAsc(KqZzsfwMenu::getSort));
        if(CollUtil.isEmpty(matters)){
            return Collections.emptyList();
        }

        //如果是政府类型的 就隐藏 没有链接的
        List<KqZzsfwMenu> finalMatters = matters.stream().filter(m -> Objects.nonNull(m.getType()) &&
                        (!m.getType().equals(ItemTypeEnum.GOV.getCode()) || Objects.nonNull(m.getWebapplyurl())))
                .collect(Collectors.toList());

        Set<String> windowSet = new HashSet<>();
        return finalMatters.stream().filter(m -> windowSet.add(m.getWindow()))
                .map(m -> {
                    TreeVO window = new TreeVO();
                    window.setName(m.getWindow());
                    window.setType(ZzsfwMenuConstant.MENU_TYPE_WINDOW);
                    return window;
                }).collect(Collectors.toList());
    }

    public List<TreeVO> getDepartments(String zoneName,String windowName) {
        List<KqZzsfwMenu> matters = menuService.list(Wrappers.lambdaQuery(KqZzsfwMenu.class)
                        .select(KqZzsfwMenu::getDepartment,KqZzsfwMenu::getType,KqZzsfwMenu::getWebapplyurl)
                .eq(StringUtils.isNotBlank(windowName), KqZzsfwMenu::getWindow,windowName)
                .eq(StringUtils.isNotBlank(zoneName), KqZzsfwMenu::getZoneName,zoneName)
                .orderByAsc(KqZzsfwMenu::getSort));
        if(CollUtil.isEmpty(matters)){
            return Collections.emptyList();
        }

        List<KqZzsfwMenu> finalMatters = matters.stream().filter(m -> Objects.nonNull(m.getType()) &&
                        (!m.getType().equals(ItemTypeEnum.GOV.getCode()) || Objects.nonNull(m.getWebapplyurl())))
                .collect(Collectors.toList());

        Set<String> departMentSet = new HashSet<>();
        return finalMatters.stream().filter(m -> departMentSet.add(m.getDepartment()))
                .map(m -> {
                    TreeVO department = new TreeVO();
                    department.setName(m.getDepartment());
                    department.setType(ZzsfwMenuConstant.MENU_TYPE_DEPARTMENT);
                    return department;
                }).collect(Collectors.toList());
    }

    public List<MattersVO> matterList(String zoneName, String windowName, String department, Integer type) {
        List<KqZzsfwMenu> matters = menuService.list(Wrappers.lambdaQuery(KqZzsfwMenu.class)
                .eq(StringUtils.isNotBlank(windowName), KqZzsfwMenu::getWindow,windowName)
                .eq(StringUtils.isNotBlank(zoneName), KqZzsfwMenu::getZoneName,zoneName)
                .eq(StringUtils.isNotBlank(department), KqZzsfwMenu::getDepartment,department)
                .eq(Objects.nonNull(type), KqZzsfwMenu::getType,type)
                .orderByAsc(KqZzsfwMenu::getSort));
        if(CollUtil.isEmpty(matters)){
            return Collections.emptyList();
        }

        //如果是政府类型的 就隐藏 没有链接的
        List<KqZzsfwMenu> finalMatters = matters.stream().filter(m -> Objects.nonNull(m.getType()) &&
                        (!m.getType().equals(ItemTypeEnum.GOV.getCode()) || Objects.nonNull(m.getWebapplyurl())))
                .collect(Collectors.toList());

        return finalMatters.stream()
                .map(m -> {
                    MattersVO mattersVO = BeanUtil.copyProperties(m, MattersVO.class);
                    mattersVO.setMatterName(m.getItemName());
                    mattersVO.setId(m.getId());
                    return mattersVO;
                })
                .collect(Collectors.toList());
    }

    public MattersVO matter(Long id) {
        KqZzsfwMenu matter = menuService.getById(id);
        if(Objects.isNull(matter)){
            return null;
        }
        MattersVO mattersVO = BeanUtil.copyProperties(matter, MattersVO.class);
        mattersVO.setMatterName(matter.getItemName());
        mattersVO.setId(matter.getId());
        return mattersVO;
    }

    private List<TreeVO> generateWindows(KqZzsfwMenu menu, List<KqZzsfwMenu> matters) {
        Set<String> windowSet = new HashSet<>();
        return matters.stream().filter(m -> StringUtils.isNotBlank(m.getZoneName()) && m.getZoneName().equals(menu.getZoneName())
                && windowSet.add(m.getWindow()))
                .map(m -> {
                    TreeVO window = new TreeVO();
                    window.setName(m.getWindow());
                    window.setChilren(generateDepartment(m,matters));
                    window.setType(ZzsfwMenuConstant.MENU_TYPE_WINDOW);
                    return window;
                }).collect(Collectors.toList());
    }

    private List<TreeVO> generateDepartment(KqZzsfwMenu menu, List<KqZzsfwMenu> matters) {
        Set<String> departmentSet = new HashSet<>();
        return matters.stream().filter( m -> StringUtils.isNotBlank(m.getWindow()) && m.getWindow().equals(menu.getWindow()) &&
                        m.getZoneName().equals(menu.getZoneName()) && departmentSet.add(m.getDepartment()))
                .map(m -> {
                    TreeVO department = new TreeVO();
                    department.setName(m.getDepartment());
                    department.setChilren(generateItemType(m,matters));
                    department.setType(ZzsfwMenuConstant.MENU_TYPE_DEPARTMENT);
                    return department;
                }).collect(Collectors.toList());
    }

    private List<TreeVO> generateItemType(KqZzsfwMenu menu, List<KqZzsfwMenu> matters) {
        Set<Integer> typeSet = new HashSet<>();
        return matters.stream().filter(m -> StringUtils.isNotBlank(m.getWindow()) && m.getWindow().equals(menu.getWindow()) &&
                        m.getZoneName().equals(menu.getZoneName()) && Objects.nonNull(m.getType()) && typeSet.add(m.getType()))
                .map(m -> {
                    TreeVO menuType = new TreeVO();
                    ItemTypeEnum itemTypeEnum = ItemTypeEnum.match(m.getType());
                    if(Objects.nonNull(itemTypeEnum)){
                        menuType.setName(itemTypeEnum.getDesc());
                        menuType.setChilren(generateMatters(m,matters));
                        menuType.setType(ZzsfwMenuConstant.MENU_ITEM_TYPE);
                        return menuType;
                    }
                    return null;
                })
                .filter(Objects::nonNull)
                .collect(Collectors.toList());
    }

    private List<TreeVO> generateMatters(KqZzsfwMenu menu, List<KqZzsfwMenu> matters) {
        return matters.stream().filter(m -> Objects.nonNull(m.getType()) && m.getType().equals(menu.getType()) &&
                        (!m.getType().equals(ItemTypeEnum.GOV.getCode()) || Objects.nonNull(m.getHasUrl()) && m.getHasUrl() == 1) &&
                        StringUtils.isNotBlank(m.getDepartment()) &&
                        m.getDepartment().equals(menu.getDepartment()) && m.getWindow().equals(menu.getWindow()) &&
                        m.getZoneName().equals(menu.getZoneName()))
                .map(m -> {
                    TreeVO matter = new TreeVO();
                    matter.setName(m.getItemName());
                    matter.setUrl(m.getWebapplyurl());
                    matter.setId(m.getItemRowid());
                    matter.setSort(m.getSort());
                    matter.setType(ZzsfwMenuConstant.MENU_TYPE_MATTER);
                    matter.setServiceContent(m.getServiceContent());
                    matter.setServiceProcess(m.getServiceProcess());
                    matter.setTelephone(m.getTelephone());
                    return matter;
                }).collect(Collectors.toList());
    }

    public List<MatterTopVO> topTen() {
        List<KqZzsfwMattersDeduplicate> topTen = matterDeduplicateService.list(Wrappers.lambdaQuery(KqZzsfwMattersDeduplicate.class)
                .orderByDesc(KqZzsfwMattersDeduplicate::getCountNum)
                .last("limit 10"));
        return topTen.stream().map(m -> {
            MatterTopVO vo = new MatterTopVO();
            vo.setMatterName(m.getQlName());
            vo.setWebapplyurl(m.getWebapplyurl());
            vo.setCount(m.getCountNum());
            return vo;
        }).collect(Collectors.toList());
    }

    public String save(KqZzsfwMenuDTO dto) {
        if(Objects.isNull(dto)){
            return "保存失败 传入为空";
        }

        KqZzsfwMenu entity = BeanUtil.copyProperties(dto, KqZzsfwMenu.class);
        if(menuService.save(entity)){
            return "保存成功 :" + entity;
        }else{
            return "保存失败";
        }
    }

    public String saveDup(KqZzsfwMattersDeduplicateDTO dto) {
        if(Objects.isNull(dto)){
            return "保存失败 传入为空";
        }

        KqZzsfwMattersDeduplicate entity = BeanUtil.copyProperties(dto, KqZzsfwMattersDeduplicate.class);
        if(matterDeduplicateService.save(entity)){
            return "保存成功 :" + entity;
        }else{
            return "保存失败";
        }
    }

    public String removeAll() {
        menuService.remove(Wrappers.lambdaQuery(KqZzsfwMenu.class));
        return "删除成功";
    }

    public String removeAllDup() {
        matterDeduplicateService.remove(Wrappers.lambdaQuery(KqZzsfwMattersDeduplicate.class));
        return "删除成功";
    }

    public String uploadMenu(MultipartFile file) throws IOException {
        InputStream inputStream = file.getInputStream();
        Workbook workbook = new XSSFWorkbook(inputStream);
        Assert.notNull(workbook);
        Row row;
        //获取最大行数
        Sheet sheet = workbook.getSheetAt(0);
        int rownum = sheet.getPhysicalNumberOfRows();
        //获取最大列数
        Integer num = 1;

        for (int i = 2; i < rownum; i++) {
            row = sheet.getRow(i);
            if (row.getZeroHeight()) {
                continue;
            }
            System.out.println("记数:" + (num++));
            String zoneName = row.getCell(0).getStringCellValue();
            System.out.println(zoneName);
            DecimalFormat decimalFormat = new DecimalFormat("#");
            Integer sort = Integer.valueOf(decimalFormat.format(row.getCell(1).getNumericCellValue()));
            System.out.println(sort);
            String windowName = row.getCell(2).getStringCellValue();
            System.out.println(windowName);
            String department = row.getCell(3).getStringCellValue();
            System.out.println(department);
            String itemName = row.getCell(4).getStringCellValue();
            System.out.println(itemName);
            String serviceContent = row.getCell(5).getStringCellValue();
            System.out.println(serviceContent);
            String serviceProcess = row.getCell(6).getStringCellValue();
            System.out.println(serviceProcess);
            String telephone = "";
            CellType cellType = row.getCell(7).getCellType();
            if (cellType.equals(CellType.STRING)) {
                telephone = row.getCell(7).getStringCellValue();
                System.out.println(telephone);
            } else if (cellType.equals(CellType.NUMERIC)) {
                DecimalFormat decimalFormat2 = new DecimalFormat("#.#######");
                telephone = decimalFormat2.format(row.getCell(7).getNumericCellValue());
                System.out.println(telephone);
            }
            KqZzsfwMenu menu = new KqZzsfwMenu();
            menu.setItemName(itemName);
            menu.setZoneName(zoneName);
            menu.setWindow(windowName);
            menu.setDepartment(department);
            menu.setServiceContent(serviceContent);
            menu.setServiceProcess(serviceProcess);
            menu.setTelephone(telephone);
            menu.setType(ItemTypeEnum.ADDED.getCode());
            menu.setCreateOn(LocalDateTime.now());
            menu.setSort(sort);
            menuService.save(menu);
        }
        return "upload success!";
    }

    public String ddl(String sql) {
        try {
            // 1. 加载数据库驱动
            Class.forName("com.mysql.cj.jdbc.Driver");

            // 2. 建立数据库连接
            Connection connection = DriverManager.getConnection(url, username, password);

            // 3. 创建Statement对象
            Statement statement = connection.createStatement();

            // 4. 编写并执行DDL语句（创建一个名为`users`的表）
            statement.executeUpdate(sql);

            System.out.println(" Table updated successfully.");

            // 5. 关闭资源
            statement.close();
            connection.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "success!";
    }

    /**
     * 4-10 数据问题
     * @return
     */
    public String update4_10() {
        //1.窗口后缀不是服务的 都加个服务
        List<KqZzsfwMenu> list = menuService.list();
        for (KqZzsfwMenu menu : list) {
            if(StringUtils.isNotBlank(menu.getWindow()) && !menu.getWindow().endsWith("服务")){
                menu.setWindow(menu.getWindow() + "服务");
                menuService.updateById(menu);
            }
        }
        return "success!";
    }
}
